@@ -1,19 +1,18 @@
-import multiprocessing
 import os
 import sys
 import threading
+import multiprocessing
 import cherrypy  # type: ignore
 import time
+import requests
+import resotolib.proc
 from functools import partial
 from queue import Queue
 from signal import SIGTERM
-from typing import List, Dict, Type, Optional, Any
-
-import requests
-
-import resotolib.proc
+from typing import List, Dict, Type, Optional, Any, Callable
+from multiprocessing.managers import SyncManager
 from resotolib.args import ArgumentParser
-from resotolib.baseplugin import BaseActionPlugin, BasePostCollectPlugin, BaseCollectorPlugin, PluginType
+from resotolib.baseplugin import BaseActionPlugin, BaseCollectorPlugin, PluginType
 from resotolib.config import Config
 from resotolib.core import add_args as core_add_args, resotocore, wait_for_resotocore
 from resotolib.core.actions import CoreActions, CoreFeedback
@@ -27,7 +26,7 @@ from resotolib.event import (
 from resotolib.jwt import add_args as jwt_add_args
 from resotolib.logger import log, setup_logger, add_args as logging_add_args
 from resotolib.core.custom_command import command_definitions
-from resotolib.proc import log_stats, increase_limits
+from resotolib.proc import log_stats
 from resotolib.types import Json
 from resotolib.web import WebServer
 from resotolib.web.metrics import WebApp
@@ -75,6 +74,9 @@ def main() -> None:
     # added their args to the arg parser
     arg_parser.parse_args()
 
+    # Register functions to detect system healthiness
+    health_conditions: Dict[str, Callable[[], bool]] = {}
+
     try:
         wait_for_resotocore(resotocore.http_uri)
     except TimeoutError as e:
@@ -93,12 +95,13 @@ def main() -> None:
         resotocore_uri=resotocore.http_uri,
         tls_data=tls_data,
     )
+    health_conditions["Config Listener"] = config.connected
 
     add_config(config, plugin_loader.all_collector_plugins())
     plugin_loader.add_plugin_config(config)
     config.load_config()
 
-    write_files_to_home_dir(config.resotoworker.write_files_to_home_dir, write_utf8_file)
+    write_files_to_home_dir(config.resotoworker.all_files_in_home_dir(), write_utf8_file)
 
     def send_request(request: requests.Request) -> requests.Response:
         prepared = request.prepare()
@@ -111,18 +114,17 @@ def main() -> None:
     core = Resotocore(send_request, config)
 
     # the multiprocessing manager is used to share data between processes
-    mp_manager = multiprocessing.Manager()
+    ctx = multiprocessing.get_context("spawn")
+    mp_manager = SyncManager(ctx=ctx)
+    mp_manager.start(initializer=resotolib.proc.increase_limits)
     core_messages: Queue[Json] = mp_manager.Queue()
 
-    collector = Collector(config, core.send_to_resotocore, core_messages)
+    collector = Collector(config, core, core_messages)
 
     # Handle Ctrl+c and other means of termination/shutdown
     resotolib.proc.initializer()
     add_event_listener(EventType.SHUTDOWN, shutdown, blocking=False)
 
-    # Try to increase nofile and nproc limits
-    increase_limits()
-
     web_server_args = {}
     if tls_data and not Config.resotoworker.no_tls:
         web_server_args = {
@@ -130,7 +132,11 @@ def main() -> None:
             "ssl_key": tls_data.key_path,
         }
     web_server = WebServer(
-        WorkerWebApp(mountpoint=Config.resotoworker.web_path, plugin_loader=plugin_loader),
+        WorkerWebApp(
+            mountpoint=Config.resotoworker.web_path,
+            plugin_loader=plugin_loader,
+            health_conditions=health_conditions,
+        ),
         web_host=Config.resotoworker.web_host,
         web_port=Config.resotoworker.web_port,
         **web_server_args,
@@ -156,6 +162,7 @@ def main() -> None:
         tls_data=tls_data,
         incoming_messages=core_messages,
     )
+    health_conditions["CoreActions"] = core_actions.connected
 
     # make tagging by collectors available out of the box
     collect_plugins: List[BaseCollectorPlugin] = plugin_loader.plugins(PluginType.COLLECTOR)  # type: ignore
@@ -186,6 +193,7 @@ def main() -> None:
     )
     core_actions.start()
     core_tasks.start()
+    health_conditions["CoreTasks"] = core_tasks.connected
 
     for plugin_class in plugin_loader.plugins(PluginType.ACTION):
         assert issubclass(plugin_class, BaseActionPlugin)
@@ -199,8 +207,9 @@ def main() -> None:
     # We wait for the shutdown Event to be set() and then end the program
     # While doing so we print the list of active threads once per 15 minutes
     shutdown_event.wait()
-    web_server.shutdown()  # type: ignore
+    web_server.shutdown()
     time.sleep(1)  # everything gets 1000ms to shutdown gracefully before we force it
+    mp_manager.shutdown()
     resotolib.proc.kill_children(SIGTERM, ensure_death=True)
     log.info("Shutdown complete")
     os._exit(0)
@@ -214,7 +223,6 @@ def core_actions_processor(
     message: Dict[str, Any],
 ) -> Optional[Dict[str, Any]]:
     collectors: List[Type[BaseCollectorPlugin]] = plugin_loader.plugins(PluginType.COLLECTOR)  # type: ignore
-    post_collectors: List[Type[BasePostCollectPlugin]] = plugin_loader.plugins(PluginType.POST_COLLECT)  # type: ignore
     # todo: clean this up
     if not isinstance(message, dict):
         log.error(f"Invalid message: {message}")
@@ -224,20 +232,21 @@ def core_actions_processor(
     data = message.get("data") or {}
     task_id: str = data.get("task")  # type: ignore
     step_name: str = data.get("step")  # type: ignore
+
     log.debug(f"Received message of kind {kind}, type {message_type}, data: {data}")
     if kind == "action":
         try:
             if message_type == "collect":
                 start_time = time.time()
-                collector.collect_and_send(collectors, post_collectors, task_id=task_id, step_name=step_name)
+                collector.collect_and_send(collectors, task_data=data)
                 run_time = int(time.time() - start_time)
                 log.info(f"Collect ran for {run_time} seconds")
             elif message_type == "cleanup":
                 if not Config.resotoworker.cleanup:
-                    log.info("Cleanup called but disabled in config" " (resotoworker.cleanup) - skipping")
+                    log.info("Cleanup called but disabled in config (resotoworker.cleanup) - skipping")
                 else:
                     if Config.resotoworker.cleanup_dry_run:
-                        log.info("Cleanup called with dry run configured" " (resotoworker.cleanup_dry_run)")
+                        log.info("Cleanup called with dry run configured (resotoworker.cleanup_dry_run)")
                     start_time = time.time()
                     feedback = CoreFeedback(task_id, step_name, "cleanup", collector.core_messages)
                     cleanup(
@@ -271,7 +280,7 @@ def core_actions_processor(
 
 
 def shutdown(event: Event) -> None:
-    reason = event.data.get("reason")
+    reason = str(event.data.get("reason"))
     emergency = event.data.get("emergency")
 
     if emergency:
