@@ -5,8 +5,7 @@ from resotolib.proc import num_default_threads
 from resotolib.baseplugin import BaseCollectorPlugin
 from resotolib.logger import log
 from attrs import define, field, frozen
-from typing import ClassVar, Optional, List, Type
-
+from typing import ClassVar, Optional, List, Type, Dict
 
 _default_collectors: List[str] = []
 
@@ -43,7 +42,7 @@ def set_default_collectors(collector_plugins: List[Type[BaseCollectorPlugin]]) -
 class HomeDirectoryFile:
     kind: ClassVar[str] = "resotoworker_home_directory_file"
     path: str = field(metadata={"description": "Path to the file"})
-    content: str = field(metadata={"description": "Content of the file"})
+    content: str = field(metadata={"description": "Content of the file", "ui-hint": "multiline"})
 
 
 @define
@@ -59,9 +58,12 @@ class ResotoWorkerConfig:
     )
     timeout: int = field(default=10800, metadata={"description": "Collection/cleanup timeout in seconds"})
     pool_size: int = field(default=5, metadata={"description": "Collector thread/process pool size"})
+    graph_sender_pool_size: int = field(
+        default=5, metadata={"description": "Maximum number of graphs to send to the core concurrently"}
+    )
     fork_process: bool = field(default=True, metadata={"description": "Use forked process instead of threads"})
     graph_merge_kind: GraphMergeKind = field(
-        default=GraphMergeKind.cloud,
+        default=GraphMergeKind.account,
         metadata={"description": "Resource kind to merge graph at (cloud or account)"},
     )
     debug_dump_json: bool = field(default=False, metadata={"description": "Dump the generated JSON data to disk"})
@@ -105,12 +107,28 @@ class ResotoWorkerConfig:
     )
     write_files_to_home_dir: List[HomeDirectoryFile] = field(
         factory=list,
+        metadata={
+            "description": "Deprecated. Use files_in_home_dir instead.",
+            "restart_required": True,
+        },
+    )
+    # optional for backwards compatibility
+    files_in_home_dir: Optional[Dict[str, str]] = field(  # type: ignore
+        factory=dict,
         metadata={
             "description": (
-                "All entries that are defined in this section are created as files on demand. "
+                "All entries that are defined in this section are created as files on demand.\n"
                 "Use this option to define .aws/credentials, .kube/config file or other "
-                "credential files that should be passed to the worker as file."
+                "credential files that should be passed to the worker as file.\n"
+                "The key is the path to the file, the value is the content of the file."
             ),
             "restart_required": True,
         },
     )
+
+    def all_files_in_home_dir(self) -> List[HomeDirectoryFile]:
+        files = self.write_files_to_home_dir.copy()
+        if self.files_in_home_dir is not None:
+            for path, content in self.files_in_home_dir.items():
+                files.append(HomeDirectoryFile(path=path, content=content))
+        return files
